
local myname, ns = ...
local _G = getfenv(0)

--=================================================
-- Upvalued functions and variables
--=================================================

local print		= _G.print
local select	= _G.select


--=================================================
-- Utility functions and constants
--=================================================

local myfullname = _G.GetAddOnMetadata(myname, "Title")
function ns.Print(...) print("|cFF33FF99".. myfullname.. "|r:", ...) end

function ns.err(msg, ...) _G.geterrorhandler()(msg:format(_G.tostringall(...)) .. " - " .. _G.time()) end
local err = ns.err

local IS_MOP = select(4, _G.GetBuildInfo()) >= 50000
local ACTIVE_QUEST_PARAMS = IS_MOP and 5 or 4

--=================================================
-- Even Dispatcher
--=================================================

local f = _G.CreateFrame("frame")
f:SetScript("OnEvent", function(self, event, ...) if ns[event] then return ns[event](ns, event, ...) end end)

function ns:RegisterEvent(...) for i=1,select("#", ...) do f:RegisterEvent((select(i, ...))) end end
function ns:UnregisterEvent(...) for i=1,select("#", ...) do f:UnregisterEvent((select(i, ...))) end end
ns.RegisterEvents, ns.UnregisterEvents = ns.RegisterEvent, ns.UnregisterEvent

--=================================================
-- Init addon
--=================================================

ns:RegisterEvent("ADDON_LOADED")
function ns:ADDON_LOADED(event, addon)
	if addon ~= myname then return end

	-- Events
	self:RegisterEvent("GOSSIP_SHOW")

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil

	if _G.IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
end


function ns:PLAYER_LOGIN()
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

--=================================================
-- Core functionality
--=================================================

-- Options to select if available
local good_options = {
	trainer = true,
	vendor = true,
}

function ns:GOSSIP_SHOW()
	-- If the shift key is pressed, show the normal gossip
	if _G.IsShiftKeyDown() then return end

	-- Turn in the firt completed quest of the list
	for i=1,_G.GetNumGossipActiveQuests() do
		local is_completed = select((i-1)*ACTIVE_QUEST_PARAMS + 4, _G.GetGossipActiveQuests())
		if is_completed then
			_G.SelectGossipActiveQuest(i)
			return
		end
	end

	-- If there is only one selectable quest
	if _G.GetNumGossipAvailableQuests() == 1 then
		_G.SelectGossipAvailableQuest(1)
		return
	end

	-- Find the other interesting stuff and oppen the corresponding option
	for i=1,_G.GetNumGossipOptions() do
		local option = select(i*2, _G.GetGossipOptions()) -- (i-1)*2 + 2
		if good_options[option] then
			_G.SelectGossipOption(i)
			return
		end
	end
end
